package com.example.micrometerdemo;

import io.micrometer.observation.Observation;
import io.micrometer.observation.ObservationRegistry;
import io.micrometer.tracing.BaggageInScope;
import io.micrometer.tracing.Tracer;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequiredArgsConstructor
@Slf4j
public class JobController {

  private final ExecutorService executorService;
  private final Tracer tracer;
  private final ObservationRegistry registry;

  private ThreadLocal<Observation> observationThreadLocal = new ThreadLocal<>();
  private ThreadLocal<Observation.Scope> scopeThreadLocal = new ThreadLocal<>();
  private ThreadLocal<BaggageInScope> baggageThreadLocal = new ThreadLocal<>();


  @PostMapping("/jobs")
  public String startNew() {
    String id = UUID.randomUUID().toString();
    executorService.submit(() -> newJobThread(id));

    return id;
  }

  private void newJobThread(String id) {
    observationThreadLocal.set(Observation.createNotStarted("Job", registry).start());
    scopeThreadLocal.set(observationThreadLocal.get().openScope());
    baggageThreadLocal.set(tracer.createBaggageInScope("jobId", id));

    try {
      List<Future<?>> futures = new ArrayList<>();
      log.info("Starting new Job");
      futures.add(executorService.submit(() -> {
        try {
          Thread.sleep(100);
        } catch (InterruptedException e) {
        }
        log.info("Step 1 finished.");
      }));

      futures.add(executorService.submit(() -> {
        try {
          Thread.sleep(10);
        } catch (InterruptedException e) {
        }
        log.info("Step 2 finished.");
      }));

      futures.add(executorService.submit(() -> {
        try {
          Thread.sleep(110);
        } catch (InterruptedException e) {
        }
        log.info("Step 3 finished.");
      }));

      futures.forEach(f -> {
        try {
          f.get();
        } catch (InterruptedException | ExecutionException e) {
        }
      });
      log.info("Finished new Job");
    } finally {
      scopeThreadLocal.get().close();
      baggageThreadLocal.get().close();
      observationThreadLocal.get().stop();
      scopeThreadLocal.remove();
      baggageThreadLocal.remove();
      observationThreadLocal.remove();
    }
  }
}
